"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateSessionKeys = exports.decryptJoinAccept = exports.decryptJoin = exports.decrypt = exports.encrypt = void 0;
const util_1 = require("./util");
const crypto_js_1 = __importDefault(require("crypto-js"));
const LORAIV = crypto_js_1.default.enc.Hex.parse("00000000000000000000000000000000");
function decrypt(payload, AppSKey, NwkSKey, fCntMSB32) {
    if (!payload.PHYPayload || !payload.FRMPayload)
        throw new Error("Payload was not defined");
    if (!fCntMSB32)
        fCntMSB32 = Buffer.alloc(2, 0);
    const blocks = Math.ceil(payload.FRMPayload.length / 16);
    const sequenceS = Buffer.alloc(16 * blocks);
    for (let block = 0; block < blocks; block++) {
        const ai = _metadataBlockAi(payload, block, fCntMSB32);
        ai.copy(sequenceS, block * 16);
    }
    const key = payload.getFPort() === 0 ? NwkSKey : AppSKey;
    if (!key || key.length !== 16)
        throw new Error("Expected a appropriate key with length 16");
    const cipherstream_base64 = crypto_js_1.default.AES.encrypt(crypto_js_1.default.enc.Hex.parse(sequenceS.toString("hex")), crypto_js_1.default.enc.Hex.parse(key.toString("hex")), {
        mode: crypto_js_1.default.mode.ECB,
        iv: LORAIV,
        padding: crypto_js_1.default.pad.NoPadding,
    });
    const cipherstream = Buffer.from(cipherstream_base64.toString(), "base64");
    const plaintextPayload = Buffer.alloc(payload.FRMPayload.length);
    for (let i = 0; i < payload.FRMPayload.length; i++) {
        const Si = cipherstream.readUInt8(i);
        plaintextPayload.writeUInt8(Si ^ payload.FRMPayload.readUInt8(i), i);
    }
    return plaintextPayload;
}
exports.decrypt = decrypt;
// Check
function decryptJoin(payload, AppKey) {
    if (!(payload === null || payload === void 0 ? void 0 : payload.MACPayloadWithMIC))
        throw new Error("Expected parsed payload to be defined");
    if (AppKey.length !== 16)
        throw new Error("Expected a appropriate key with length 16");
    const cipherstream = crypto_js_1.default.AES.decrypt(payload.MACPayloadWithMIC.toString("base64"), crypto_js_1.default.enc.Hex.parse(AppKey.toString("hex")), {
        mode: crypto_js_1.default.mode.ECB,
        padding: crypto_js_1.default.pad.NoPadding,
    });
    return Buffer.from(cipherstream.toString(), "hex");
}
exports.decryptJoin = decryptJoin;
function generateSessionKeys(AppKey, NetId, AppNonce, DevNonce) {
    if (AppKey.length !== 16)
        throw new Error("Expected a AppKey with length 16");
    if (NetId.length !== 3)
        throw new Error("Expected a NetId with length 3");
    if (AppNonce.length !== 3)
        throw new Error("Expected a AppNonce with length 3");
    if (DevNonce.length !== 2)
        throw new Error("Expected a DevNonce with length 2");
    const nwkSKeyNonce = Buffer.concat([
        Buffer.from("01", "hex"),
        util_1.reverseBuffer(AppNonce),
        util_1.reverseBuffer(NetId),
        util_1.reverseBuffer(DevNonce),
        Buffer.from("00000000000000", "hex"),
    ]);
    const nwkSKey_base64 = crypto_js_1.default.AES.encrypt(crypto_js_1.default.enc.Hex.parse(nwkSKeyNonce.toString("hex")), crypto_js_1.default.enc.Hex.parse(AppKey.toString("hex")), {
        mode: crypto_js_1.default.mode.ECB,
        padding: crypto_js_1.default.pad.NoPadding,
    });
    const NwkSKey = Buffer.from(nwkSKey_base64.toString(), "base64");
    const appSKeyNonce = Buffer.concat([
        Buffer.from("02", "hex"),
        util_1.reverseBuffer(AppNonce),
        util_1.reverseBuffer(NetId),
        util_1.reverseBuffer(DevNonce),
        Buffer.from("00000000000000", "hex"),
    ]);
    const appSKey_base64 = crypto_js_1.default.AES.encrypt(crypto_js_1.default.enc.Hex.parse(appSKeyNonce.toString("hex")), crypto_js_1.default.enc.Hex.parse(AppKey.toString("hex")), {
        mode: crypto_js_1.default.mode.ECB,
        padding: crypto_js_1.default.pad.NoPadding,
    });
    const AppSKey = Buffer.from(appSKey_base64.toString(), "base64");
    return { AppSKey, NwkSKey };
}
exports.generateSessionKeys = generateSessionKeys;
function encrypt(buffer, key) {
    // CHECK
    const ciphertext = crypto_js_1.default.AES.encrypt(crypto_js_1.default.enc.Hex.parse(buffer.toString("hex")), crypto_js_1.default.enc.Hex.parse(key.toString("hex")), {
        mode: crypto_js_1.default.mode.ECB,
        iv: LORAIV,
        padding: crypto_js_1.default.pad.NoPadding,
    }).ciphertext.toString(crypto_js_1.default.enc.Hex);
    return Buffer.from(ciphertext, "hex");
}
exports.encrypt = encrypt;
function decryptJoinAccept(payload, appKey) {
    const payloadBuffer = payload.PHYPayload || Buffer.alloc(0);
    // Check
    const mhdr = payloadBuffer.slice(0, 1);
    const joinAccept = encrypt(payloadBuffer.slice(1), appKey);
    return Buffer.concat([mhdr, joinAccept]);
}
exports.decryptJoinAccept = decryptJoinAccept;
// Encrypt stream mixes in metadata blocks, as Ai =
//   0x01
//   0x00 0x00 0x00 0x00
//   direction-uplink/downlink [1]
//   DevAddr [4]
//   FCnt as 32-bit, lsb first [4]
//   0x00
//   counter = i [1]
function _metadataBlockAi(payload, blockNumber, fCntMSB32) {
    if (!fCntMSB32)
        fCntMSB32 = Buffer.alloc(2);
    let direction;
    if (payload.getDir() == "up") {
        direction = Buffer.alloc(1, 0);
    }
    else if (payload.getDir() == "down") {
        direction = Buffer.alloc(1, 1);
    }
    else {
        throw new Error("Decrypt error: expecting direction to be either 'up' or 'down'");
    }
    if (!payload.DevAddr)
        throw new Error("Decrypt error: DevAddr not defined'");
    if (!payload.FCnt)
        throw new Error("Decrypt error: FCnt not defined'");
    const aiBuffer = Buffer.concat([
        Buffer.from("0100000000", "hex"),
        direction,
        util_1.reverseBuffer(payload.DevAddr),
        util_1.reverseBuffer(payload.FCnt),
        fCntMSB32,
        Buffer.alloc(1, 0),
        Buffer.alloc(1, blockNumber + 1),
    ]);
    return aiBuffer;
}
//# sourceMappingURL=crypto.js.map